{Writer}  = require "fstream"
path      = require "path"
awssum    = require 'awssum'
amazon    = awssum.load 'amazon/amazon'
s3Service = awssum.load('amazon/s3').S3
mime      = require('mime')

class DirWriter extends Writer 
  constructor: (@params, @rel="") ->
    s3params =
        accessKeyId: @params.accessKeyId,  
        secretAccessKey: @params.secretAccessKey, 
        region: amazon[@params.region], 
    @s3 = new s3Service(s3params)
    @_pushing = false
    @_ended = false
    @_buffer = []

  add: (rel, entry) ->
    unless entry?
      entry = rel
      rel = ""

    if entry.type == "Directory"
      newRel = entry.path.substr(-1*((entry.basename || entry.path).length + @rel.length + 1))
      entry.pipe(new DirWriter(@params, newRel))
    else
      @_buffer.push(entry)
      @_process()

    return @_buffer.length is 0

  _process: () ->
    if @_pushing or @_buffer.length == 0 
      return
    

    @_pushing = true
    entry = @_buffer.shift()
    relPath = entry.path.substr(-1*((entry.basename || entry.path).length + @rel.length + 1))
    @emit "entry", entry

    newObject =
      BucketName: @params.bucket
      ContentLength: entry.props.size
      Body: entry
      ObjectName: path.join(@params.baseDir, relPath)
      ContentType: mime.lookup(relPath)

    newObject.ObjectName = newObject.ObjectName.replace(/\\/g, "\/")  if process.platform == "win32"

    entry.on "end", ->
      _this._pushing = false;
      _this._process();

    @s3.PutObject newObject, (err, r) => 
                                if err
                                  @emit "error", err 
                                else
                                  @_pushing = false
                                  entry.resume()
                                  @emit "drain"
                                  @_process()

  write: () -> true

  end: () ->
    unless @_ended
      @_process()
    @_ended = true

module.exports = DirWriter